#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
AudioPluginAudioProcessorEditor::AudioPluginAudioProcessorEditor (AudioPluginAudioProcessor& p)
    : AudioProcessorEditor (&p), processorRef (p)
{
    setSize (800, 200);

    addAndMakeVisible (button);
    addAndMakeVisible (label);

    updateText();

    label.setJustificationType (juce::Justification::centred);
    button.onClick = [this] { processorRef.useNextChannel(); };

    startTimerHz (60);
}

void AudioPluginAudioProcessorEditor::paint (juce::Graphics& g)
{
    g.fillAll (juce::Colours::black);

    juce::FlexBox box;

    for (auto& i : processorRef.inputLevels)
    {
        juce::ignoreUnused (i);
        box.items.add (juce::FlexItem().withFlex (1.0f).withMargin (4.0f));
    }

    box.performLayout (getLocalBounds().removeFromTop (getLocalBounds().proportionOfHeight (0.5f)));

    for (auto i = 0; i < (int) processorRef.inputLevels.size(); ++i)
    {
        const auto minDb = -50.0f;
        const auto db = juce::Decibels::gainToDecibels (processorRef.inputLevels[(size_t) i].load(), minDb);
        const auto scaledDb = juce::jmap (db, minDb, 12.0f, 0.0f, 1.0f);
        const auto fullBox = box.items[i].currentBounds;
        const auto textHeight = 20.0f;

        g.setColour (juce::Colours::cyan);
        g.fillRect (fullBox.withHeight (scaledDb * (fullBox.getHeight() - textHeight)).withBottomY (fullBox.getBottom()));

        const auto channelType = getTypeOfAbsoluteChannel (true, i);

        g.setColour (juce::Colours::white);
        g.drawText (juce::AudioChannelSet::getAbbreviatedChannelTypeName (channelType),
                    fullBox.withHeight (textHeight),
                    juce::Justification::centred);
    }
}

void AudioPluginAudioProcessorEditor::resized()
{
    auto b = getLocalBounds();
    b.removeFromTop (b.proportionOfHeight (0.5f));
    button.setBounds (b.removeFromTop (b.proportionOfHeight (0.5f)).withSizeKeepingCentre (200, 24));
    label.setBounds (b);
}

void AudioPluginAudioProcessorEditor::updateText()
{
    label.setText (juce::AudioChannelSet::getChannelTypeName (getTypeOfAbsoluteChannel (false, processorRef.getChannel())), juce::dontSendNotification);
}

void AudioPluginAudioProcessorEditor::timerCallback() { repaint(); }
